type edge = record
              v1, v2: vertex; (* undirected *)
              length: ... (* integer or real *) 
            end;
     tree = array [1..MaxVertexMinus1] of edge;

procedure Kruskal(var G: graph; var mst: tree);
  (* The procedure assumes that the graph is connected. *)

  (* Program assumes a Union-Find package (for holding vertices) with operations
       procedure CreateUF(N: integer), procedure Union(i, j: vertex), and
       function Find(i: vertex): vertex.
     Program assumes a min-heap package (for holding edges) with operations
       procedure DeleteMin(var H: heap) and procedure Heapify(var H: heap).
     For reasons of efficiency, procedure Kruskal has direct access to the heap. *)

  label 99;
  var H: heap;
      s1, s2: vertex;
      count: integer; (* number of edges included in minimum spanning tree *)
  begin
    (* Initially each vertex is its own (degenerate) tree and the heap contains
       all the edges. *)
    CreateUF(G.size);
    ... (* Scan the adjacency lists of G, placing each edge into the heap once. *)
    Heapify(H);

    (* Remove edges from the heap until N-1 are accepted. *)
    count := 0;
    while true do
      begin
        s1 := Find(H.elts[1].v1); s2 := Find(H.elts[1].v2); (* FindMin *)
        if s1 <> s2
          then begin (* This edge does not form a cycle -- include it. *)
                 count := count + 1;
                 mst[count] := H.elts[1];
                 if count = G.size - 1 then goto 99; (* return *)
                 Union(s1,s2) (* Merge the two trees. *)
               end;
        (* Replace the minimum element of the heap with an allowable edge. *)
        while Find(H.elts[H.size].v1) = Find(H.elts[H.size].v2) do
          H.size := H.size - 1;
        DeleteMin(H) (* no need to return minimum element *)
      end;
99:
  end; (* Kruskal *)
